@@ -94,7 +94,7 @@ sub cmd {
             nofork       => "don't fork"
             background   => "return $jobid if set, or redirect otherwise"
             clean        => "remove job after displaying it if true"
-            render       => "set to a true value to render page immediatly"
+            render       => "set to a true value to render page immediately"
             show_output  => show console with output
             log_archive  => store stdout/stderr with this filename
         }
@@ -165,6 +165,9 @@ sub perl {
             Thruk::Utils::IO::write($dir."/rc", ($rc ? 0 : 1));
             Thruk::Utils::IO::write($dir."/perl_res", (defined $rc && ref $rc eq '') ? Thruk::Utils::Encode::encode_utf8($rc) : "", undef, 1);
 
+            my $res_ctype = $c->res->content_type();
+            $c->stash->{'res_ctype'} = $res_ctype if $res_ctype;
+
             wrap_prefix_output_stop();
             close(*STDOUT);
             close(*STDERR);
@@ -706,10 +709,11 @@ wait for this job to finish
 sub wait_for_job {
     my($c, $job, $timeout) = @_;
     $timeout = 3 unless $timeout;
+    my $wait_until = time() + $timeout;
 
     my($is_running, $time) = get_status($c, $job);
     $c->stats->profile(begin => "job_page waiting for job: ".$job);
-    while($is_running and $time < $timeout) {
+    while($is_running && time() < $wait_until) {
         Time::HiRes::sleep(0.1) if $time <  1;
         Time::HiRes::sleep(0.3) if $time >= 1;
         ($is_running, $time) = get_status($c, $job);
@@ -972,6 +976,9 @@ sub do_parent_stuff {
         if(defined $conf->{'initwait'}) {
             $append = '&initwait='.$conf->{'initwait'};
         }
+        if($c->req->parameters->{'json'}) {
+            return $c->redirect_to($c->stash->{'url_prefix'}."r/thruk/job/".$id."/output", 1);
+        }
         return $c->redirect_to($c->stash->{'url_prefix'}."cgi-bin/job.cgi?job=".$id.$append);
     }
     return $id;
